'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.HashLink = HashLink;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactRouterDom = require('react-router-dom');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var hashFragment = '';
var observer = null;
var asyncTimer = null;

function reset() {
  hashFragment = '';
  if (observer !== null) observer.disconnect();
  if (asyncTimer !== null) {
    window.clearTimeout(asyncTimer);
    asyncTimer = null;
  }
}

function getElAndScroll() {
  var element = document.getElementById(hashFragment);
  if (element !== null) {
    element.scrollIntoView();
    reset();
    return true;
  }
  return false;
}

function setupMutationObserver() {
  observer = new MutationObserver(getElAndScroll);
}

function hashLinkScroll() {
  // Push onto callback queue so it runs after the DOM is updated
  setTimeout(function () {
    if (getElAndScroll() === false) {
      if (observer === null) setupMutationObserver();
      observer.observe(document, { attributes: true, childList: true, subtree: true });
      if (asyncTimer !== null) window.clearTimeout(asyncTimer);
      // if the element doesn't show up in 10 seconds, stop checking
      asyncTimer = window.setTimeout(function () {
        reset();
      }, 10000);
    }
  }, 0);
}

function HashLink(props) {
  function handleClick(e) {
    if (props.onClick) props.onClick(e);
    var hash = '';
    if (typeof props.to === 'string') {
      hash = props.to.split('#').slice(1).join('#');
    } else if (_typeof(props.to) === 'object' && typeof props.to.hash === 'string') {
      hash = props.to.hash.replace('#', '');
    }
    if (hash !== '') {
      hashFragment = hash;
      hashLinkScroll();
    }
  }
  return _react2.default.createElement(
    _reactRouterDom.Link,
    _extends({}, props, { onClick: handleClick }),
    props.children
  );
}

HashLink.propTypes = {
  onClick: _react.PropTypes.func,
  children: _react.PropTypes.node,
  to: _react.PropTypes.oneOfType([_react.PropTypes.string, _react.PropTypes.object])
};